from internetarchive import get_item

from Alexandria.general.console import print_color

from anchorage.anchor_utils.shell import shell


def add(url, archives=None, overwrite=False):
    """
    Archive a website in one of the four archives supported by Archive Now (archivenow).

    :param url: URL of website to be archived.
    :param archives: List or string of flags specifying in which archives to save the website.
                     Available flags:
                        - https://pypi.org/project/archivenow/
                     Example:
                        "all"
                        "--ia --is"
                        ["--ia", "--is"]
    :param overwrite: Archive URL even if it's already present in the Internet Archive.
    """

    def upload(url):
        flags = ' '.join(archives) if archives is list else archives if not isinstance(archives, type(None)) else ""
        try:
            copy = shell(f"archivenow {flags} {url}")
            return copy.stdout, copy.stderr
        except:
            print("Error archiving: ", end="")
            print_color(url, "red")

    if get_item(url).exists:
        if overwrite:
            return upload(url)
        else:
            return "Bookmark already present in the Internet Archive"
    else:
        return upload(url)



import os
import platform


def operating_system():
    """
    :return: Operating system of host machine.
    """
    return platform.system().lower()


def home():
    """
    :return: Directory of file currently being run.
    """
    return os.environ['HOME']



from pyfiglet import Figlet

from Alexandria.general.console import print_color

from anchorage.anchor_utils.system import operating_system


def smart_print_color(text, color, **kwargs):
    """
    Print in color in UNIX terminals, and use the regular `print` function
    elsewhere.

    :param text: Text to be printed to screen
    :param color: Text color
    :param kwargs: Any further keyword arguments for the `print` function
    """
    if operating_system() in ["linux", "macos"]:
        print_color(text, color, **kwargs)
    else:
        print(text, **kwargs)


def newline():
    """
    Print an empty line. Purely for aesthetic reasons.
    """
    print("")


def title(text="Anchorage", font="big", color="yellow"):
    """
    Generates the Anchorage CLI title.

    :param text: Text to be rendered
    :param font: Figlet font to render the title
    :param color: Title color
    """
    f = Figlet(font=font)
    smart_print_color(f.renderText(text), color)


class colors:
    """
    ANSI color codes for use with `tqdm` to generate colored progress bars.
    """

    reset = "\033[0m"

    # Black
    fgBlack = "\033[30m"
    fgBrightBlack = "\033[30;1m"
    bgBlack = "\033[40m"
    bgBrightBlack = "\033[40;1m"

    # Red
    fgRed = "\033[31m"
    fgBrightRed = "\033[31;1m"
    bgRed = "\033[41m"
    bgBrightRed = "\033[41;1m"

    # Green
    fgGreen = "\033[32m"
    fgBrightGreen = "\033[32;1m"
    bgGreen = "\033[42m"
    bgBrightGreen = "\033[42;1m"

    # Yellow
    fgYellow = "\033[33m"
    fgBrightYellow = "\033[33;1m"
    bgYellow = "\033[43m"
    bgBrightYellow = "\033[43;1m"

    # Blue
    fgBlue = "\033[34m"
    fgBrightBlue = "\033[34;1m"
    bgBlue = "\033[44m"
    bgBrightBlue = "\033[44;1m"

    # Magenta
    fgMagenta = "\033[35m"
    fgBrightMagenta = "\033[35;1m"
    bgMagenta = "\033[45m"
    bgBrightMagenta = "\033[45;1m"

    # Cyan
    fgCyan = "\033[36m"
    fgBrightCyan = "\033[36;1m"
    bgCyan = "\033[46m"
    bgBrightCyan = "\033[46;1m"

    # White
    fgWhite = "\033[37m"
    fgBrightWhite = "\033[37;1m"
    bgWhite = "\033[47m"
    bgBrightWhite = "\033[47;1m"



"""
Anchorage is a Python package which provides methods to
archive your bookmark collection in bulk simply and
effectively.

Anchorage can be used programmatically or using its CLI.

.. include:: ./documentation.md
.. include:: ../docs/anchorage_flow.svg
"""

from anchorage.anchor_infrs.infrastructure import init
from anchorage.bookmarks import bookmarks, path, load
from anchorage.anchor import anchor_locally, anchor_online
from anchorage.anchor_tools.local import add as add_local, server
from anchorage.anchor_tools.online import add as add_online



from anchorage.bookmarks import bookmarks
from anchorage.anchor_tools.local import create_archive, add as add_local, server
from anchorage.anchor_tools.online import add as add_online


def anchor_locally(collection,
                   archive="./anchor"):
    """
    Archive all bookmarked pages in a local ArchiveBox archive.

    :param collection: Bookmark JSON file or __bookmark__ class instance.
    :param archive: Full path of chosen archive. Defaults to "/anchor".
    """

    create_archive(archive)

    if isinstance(collection, bookmarks):
        err = collection.loop(lambda k, v: add_local(url=v["url"]),
                              pb_label="ARCHIVING",
                              suppress_output=True)
    else:
        err = bookmarks(collection).loop(lambda k, v: add_local(url=v["url"]),
                                         pb_label="ARCHIVING",
                                         suppress_output=True)

    log_anchorage(err)


def anchor_online(collection):
    """
    Anchor all bookmarks in an online archive.

    :param collection: Bookmark JSON file or __bookmark__ class instance.
    """

    if isinstance(collection, bookmarks):
        err = collection.loop(lambda k, v: add_online(url=v["url"]),
                              pb_label="ARCHIVING",
                              suppress_output=True)
    else:
        err = bookmarks(collection).loop(lambda k, v: add_online(url=v["url"]),
                                         pb_label="ARCHIVING",
                                         suppress_output=True)

    log_anchorage(err)


def log_anchorage(error_log):
    """
    TODO: Save error log to ~/anchorage/error_log.toml

    :param error_log: List of bookmarks for which the upload process has failed.
    """
    print(error_log)


def anchor_log_locally(archive="anchor"):
    """
    TODO: Attempt anchor of all bookmarks in the current error log.
    """

    create_archive(archive)

    pass


def anchor_log_online():
    """
    TODO: Attempt anchor of all bookmarks in the current error log.
    """
    pass



import unittest

from anchorage.bookmarks import bookmarks, load, export, path

from Alexandria.general.console import print_color


class BookmarkTests(unittest.TestCase):

    def test_path(self):
        return path("edge beta")

    def test_export(self):
        return export(self.test_path(), "tests/bookmarks/exports")

    def test_load_from_browser(self):
        return load(self.test_path())

    def test_load_from_json(self):
        return load("tests/bookmarks/test_sample.json")

    def test_load_from_jsonlz4(self):
        return load("tests/bookmarks/test_sample.jsonlz4")

    def test_bookmark_processing(self):
        bm_chr = bookmarks(self.test_load_from_json(),
                           drop_local_files=False,
                           drop_duplicate_urls=False,
                           drop_dirs=False)
        bm_ffx = bookmarks(self.test_load_from_jsonlz4(),
                           drop_local_files=False,
                           drop_duplicate_urls=False,
                           drop_dirs=False)
        assert len(bm_chr.bookmarks) == 1954
        assert len(bm_ffx.bookmarks) == 1954

    def test_tags(self):
        print(bookmarks(self.test_load_from_browser()).bookmarks)

    def test_filter_string(self):
        print(bookmarks(self.test_load_from_browser(),
                        drop_dirs=['Favorites bar']))

    def test_filter_substring(self):
        print(bookmarks(self.test_load_from_browser(),
                        drop_dirs_subs=['Favorites']))

    def test_filter_regex(self):
        print(bookmarks(self.test_load_from_browser(),
                        drop_names_regex=r'[aA]'))               # Match all names including 'a' or 'A'

    def test_loop(self):
        bookmarks(load("tests/bookmarks/test_sample.json")).loop(lambda k, v: print_color((k, v), "green"),
                                                                 pb=False,
                                                                 suppress_output=False
                                                                 )



import os
import re
import sys
import shutil
import datetime
import rapidjson
from tqdm import tqdm

from Alexandria.general.project import root

from anchorage.anchor_infrs.infrastructure import init, read_config
from anchorage.anchor_utils.aesthetic import colors
from anchorage.anchor_utils.system import operating_system
from anchorage.anchor_utils.shell import suppress_stdout
from anchorage.anchor_utils.file_conversion import JSONLZ4_to_JSON
from anchorage.anchor_utils.aesthetic import smart_print_color


def loc(overwrite=False):
    if overwrite:
        init(True)
    return read_config()


def path(browser, overwrite=False):
    """
    :return: Path of browser of choice Bookmarks file
    """
    if browser == "firefox":
        o_sys = operating_system()
        preamble = ("\\".join(str(os.getenv("APPDATA")).split("\\")[:-1]) + "\\") if o_sys == "windows" else ""
        dir_path = preamble + loc(overwrite)[browser][o_sys].replace("/", "\\")
        most_recent = datetime.datetime(1970, 1, 1)
        for filename in os.listdir(dir_path):
            file_date = datetime.datetime.strptime(filename[10:20], "%Y-%m-%d")
            if file_date > most_recent:
                most_recent = file_date
                bk_path = dir_path + "\\" + filename
    else:
        o_sys = operating_system()
        preamble = ("\\".join(str(os.getenv("APPDATA")).split("\\")[:-1])+"\\") if o_sys == "windows" else ""
        bk_path = preamble + loc(overwrite)[browser][o_sys].replace("/", "\\")
    return bk_path


def load(path):
    """
    :param path: Path to Bookmark JSON or JSONLZ4 file
    :return: Parsed JSON file
    """
    if path.split('\\')[-1].find('json') != -1:                    # Check if the JSON extension is present in path
        ext = path[len(path)-path[::-1].find('.'):].lower()        # If so obtain full extension
        if ext == "json":                                          # If full extension = JSON load normally
            return rapidjson.load(open(path, encoding="utf8"))
        elif ext == "jsonlz4":                                     # IF full extension = JSONLZ4 create bookmark
            d = rapidjson.loads(JSONLZ4_to_JSON(path))              # dictionary appropriately
            bm_list = d['children'][0]['children']
            bm_dirs = [d['name'] for d in bm_list]
            bm_dics = [d for d in bm_list]
            return {'roots': dict(zip(bm_dirs, bm_dics))}
    else:                                                          # If no extension is present, the file is assumed
        return rapidjson.load(open(path, encoding="utf8"))         # to be a JSON file (the case for Chromium browsers)


def export(path, dest=root()):
    """
    :param path: Bookmark JSON path
    :param dest: Destination directory INCLUDING THE FILENAME
    :return: Export bookmark JSON file to the project's root directory
    """
    date = datetime.datetime.today().strftime('%Y_%m_%d')
    filename = f"bookmarks_{date}" + (".jsonzl4" if ".jsonlz4" in path else ".json")
    shutil.copyfile(path, os.path.join(dest, filename))
    return dest


class bookmarks:

    def __init__(self, bookmark_dict,
                 drop_duplicate_urls=True,
                 drop_local_files=True,
                 drop_dirs=None,
                 drop_names=None,
                 drop_urls=None,
                 drop_dirs_subs=None,
                 drop_urls_subs=None,
                 drop_names_subs=None,
                 drop_dirs_regex=None,
                 drop_urls_regex=None,
                 drop_names_regex=None,
                 ):
        """
        Prepare bookmarks for archiving.

        Instance attributes:

            self.links: dictionary of - Name: Dict - pairs, where

                Dict: {"url": url,
                       "tags": [tag_1, tag_2, ..., tag_n]}

            self.tags: list containing the ordered names of the parent folders of each link.

        :param bookmark_dict: Bookmark JSON file.
        :param drop_local_files: Remove local file bookmarks from bookmark list.
        :param drop_duplicate_urls: Remove duplicated URLs.
        :param drop_dirs: Directories from which no bookmarks are to be archived.
        """

        content = bookmark_dict['roots']

        self.bookmarks = {}
        self.tags = []
        self.n_dirs = 0

        # Conduct iteration
        for directory in content:
            self.search_dict(content[directory])        # Iteration
            self.tags = []                              # "Navigate" back to root
            self.n_dirs += 1                            # Keep track of the number of directories

        # Drop duplicate URLs
        if drop_duplicate_urls:
            self.drop_duplicate_urls()

        # Filters
        if drop_local_files:                                                    # Local files
            self.drop_local_files()
        if drop_dirs:                                                           # String    - Directories
            self.drop_string(target=drop_dirs, what='dir')
        if drop_names:                                                          # String    - Names
            self.drop_string(target=drop_names, what='name')
        if drop_urls:                                                           # String    - URLs
            self.drop_string(target=drop_dirs, what='url')
        if drop_dirs_subs:                                                      # Substring - Directories
            self.drop_substring(target=drop_dirs_subs, what='dir')
        if drop_names_subs:                                                     # Substring - Names
            self.drop_substring(target=drop_names_subs, what='name')
        if drop_urls_subs:                                                      # Substring - URLs
            self.drop_substring(target=drop_urls_subs, what='url')
        if drop_dirs_regex:                                                     # Regex     - Directories
            self.drop_regex(regex=drop_dirs_regex, what='dir')
        if drop_names_regex:                                                    # Regex     - Names
            self.drop_regex(regex=drop_names_regex, what='name')
        if drop_urls_regex:                                                     # Regex     - URLs
            self.drop_regex(regex=drop_urls_regex, what='url')

    def search_dict(self, dictionary):
        """
        Reduce depth-n bookmark dictionary to depth-1 dictionary of -Name: Dict - pairs, where

            Dict: {"url": url,
                   "tags": tag_list}

        and tag_list is a list containing the ordered names of the parent folders of each link.

        :param dictionary: Bookmark dictionary from which - Name: [link, tags] - pairs are to be extracted.
        :return: Depth-1 dictionary of - Name: [link, tags] - pairs.
        """
        if "children" in dictionary.keys():
            self.tags.append(dictionary['name'])            # Append directory name to tag list
            self.n_dirs += 1                                # Keep track of the number of directories
            self.search_children(dictionary['children'])    # Search through children
            self.tag_backtrack(dictionary['name'])          # After search is over, "navigate" back to parent directory

        elif 'url' in dictionary.keys():
            # Conduct a regex search for the name of the bookmark among all those previously found.
            #
            # Avoids a name conflict caused by the nth duplicate with n>1 due to the first
            # duplicate having already a different name to the first one, as
            #
            #       name of first duplicate = <name> ::anchorage name duplicate:: 2
            #
            n_rep = len(list(filter(re.compile(f'{re.escape(dictionary["name"])}').match,
                                    list(self.bookmarks.keys()))
                             ))
            #   1. re.escape
            #          Avoid issues with bookmarks with special regex sequences in their name
            #   2. re.compile(<bookmark name>
            #          Match any string with the entire bookmark name in it

            if n_rep > 0:
                key = dictionary['name'] + f" ::anchorage name duplicate:: {n_rep+1}"
            else:
                key = dictionary['name']

            self.bookmarks[key] = {'url': dictionary['url'],
                                   'tags': self.tags.copy()}
        else:
            pass        # Account for possibly empty bookmark folders

    def search_children(self, children):
        """
        Iterate over children.

        :param children: List entries in bookmark dictionary, which are by custom named "children" in
                         bookmark dictionaries.
        """
        for child in children:
            self.search_dict(child)

    def tag_backtrack(self, tag):
        """
        Remove tracked tags deeper than input tag.
            For a series of nested dictionaries (D) which may contain "children" lists (c)

                D
                |_ c[dir_name1]             tags = [dir_name1]
                   |_ D
                   |  |_ c[dirname_2]       tags = [dir_name1, dir_name2]
                   |     |_ D
                   |_ D                  !! tags = [dir_name1]

            It is necessary to be able to "navigate" to the current parent directory after
            iterating through any subdirectories.

            The "path" to each link is kept in an ordered list. To "navigate" back after
            iterating through any previously subdirectories, all elements in the tag list
            after the specified (parent) tag are eliminated.
        """
        try:
            inv_index = self.tags[::-1].index(tag)
            index = len(self.tags) - 1 - inv_index
            self.tags = self.tags[:index]
        except ValueError:
            self.tags = [tag]

    def drop_duplicate_urls(self):
        links = []
        for key, value in self.bookmarks.copy().items():
            if value['url'] in links:
                del self.bookmarks[key]
            else:
                links.append(value['url'])

    def drop_local_files(self):
        for key, value in self.bookmarks.copy().items():
            if "http" not in value["url"][:4]:
                del self.bookmarks[key]

    def drop_string(self, target, what):
        if isinstance(target, str):
            target = [target]

        # What
        if what == 'dir':
            target_key = 'tags'
        elif what == 'name':
            target_key = 'name'
        elif what == 'url':
            target_key = 'url'
        else:
            smart_print_color(f'Wrong filter target: {what}', 'red')
            sys.exit()

        for key, value in self.bookmarks.copy().items():
            if list(set(target) & set(list([key] if target_key == 'name' else value[target_key]))):
                del self.bookmarks[key]

    def drop_substring(self, target, what):
        if isinstance(target, str):
            target = [target]

        # What
        if what == 'dir':
            target_key = 'tags'
        elif what == 'name':
            target_key = 'name'
        elif what == 'url':
            target_key = 'url'
        else:
            smart_print_color(f'Wrong filter target: {what}', 'red')
            sys.exit()

        i = 0
        for key, value in self.bookmarks.copy().items():
            rm = False
            while not rm:
                for string in key if target_key == 'name' else value[target_key]:
                    for substring in target:
                        if substring in string:
                            rm = True
                break
            if rm:
                del self.bookmarks[key]

    def drop_regex(self, regex, what):

        r = re.compile(regex)

        # What
        if what == 'dir':
            target_key = 'tags'
        elif what == 'name':
            target_key = 'name'
        elif what == 'url':
            target_key = 'url'
        else:
            smart_print_color(f'Wrong filter target: {what}', 'red')
            sys.exit()

        for key, value in self.bookmarks.copy().items():
            rm = False
            while not rm:
                for string in key if target_key == 'name' else value[target_key]:
                    if r.match(string):
                        rm = True
                break
            if rm:
                del self.bookmarks[key]

    def loop(self, f,
             pb=True,
             pb_label=None,
             pb_leave=True,
             pb_width=110,
             suppress_output=False
             ):
        """
        :param f: Function - To be run on each entry of the bookmark dictionary.
        :param pb: Boolean - True to visualize progress with tqdm progress bar.
        :param pb_label: Str - Progress bar label.
        :param pb_leave: Boolean - False to remove progress bar from screen after completion.
        :param pb_width: N - Width in char of the progress bar.
        :param suppress_output: Suppress output of the provided function.
        :return: List with all [key, value] pairs for which `f` execution resulted in an error.
        """

        e = []

        if pb:                                          # Create tqdm progress bar if specified
            pgr = tqdm(self.bookmarks.items(),
                       ncols=pb_width,
                       position=0,
                       leave=pb_leave,
                       desc=pb_label,
                       bar_format="{l_bar}"
                                  "%s{bar}"
                                  "%s| {n_fmt}/{total_fmt} "
                                  "[{elapsed}<%s{remaining}%s,"
                                  " {rate_fmt}{postfix}]" % (colors.fgYellow, colors.reset,
                                                             colors.fgRed, colors.reset)
                       )

        for key, value in self.bookmarks.items():

            try:                                        # Attempt to run provided function on dictionary
                if suppress_output:
                    with suppress_stdout():             # Suppress function output if so specified
                        f(key, value)
                else:
                    f(key, value)
            except:                                     # Error: add entry to error list
                e.append([key, value])

            if pb:                                      # Update progress bar if in use
                pgr.update()

        print("\r" + " "*pb_width, end="\r")            # Clean console from debris left by tqdm

        return e

    def __repr__(self):
        """
        :return: Name and URL of all bookmarks, as well as the total number of bookmarks
                 and directories in the collection.
        """
        links = []
        for key, value in self.bookmarks.items():
            links.append([key, value["url"]])

        lstr = "\n".join(" ".join(name_link) for name_link in links)
        info = f'\n\nFound: {len(self.bookmarks)} links and {self.n_dirs} directories.'

        return lstr + info





#!/usr/run/env python37

from __future__ import print_function, unicode_literals

import sys

from anchorage.bookmarks import bookmarks, path, load
from anchorage.anchor import anchor_online, anchor_locally
from anchorage.anchor_infrs.infrastructure import init, read_config
from anchorage.anchor_utils.shell import shell
from anchorage.anchor_utils.regex import expr_check
from anchorage.anchor_utils.aesthetic import smart_print_color, newline, title


try:
    from PyInquirer import style_from_dict, Token, prompt
except ImportError:
    print("PyInquirer import error: prompt-toolkits version != 1.0.14")
    print("Installing prompt-toolkit==1.0.14")
    shell("pip install prompt-toolkit==1.0.14")
    sys.exit("Success! Please run again")


def interface():
    """
    # CLI interface for the Anchorage library.

    1. Dependency check
        - pip install --upgrade anchorage
        - check_install
    2. Browser choice
    3. Bookmark filter
        - Local files
        - String match
            - Directories
            - Bookmark name
            - Bookmark URL
        - Substring match
            - Directories
            - Bookmark name
            - Bookmark URL
        - Regex match
            - Directories
            - Bookmark name
            - Bookmark URL
        - *Duplicate URLs are excluded by default*
    4. Local or online archive choice
        - Online
        - Local
            - Local archive directory input
    """

    # Title
    print('\nWelcome to the')
    title("Anchorage  cli")
    print("                                        Antonio Lopez Rivera, 2021\n")

    # CLI interface look
    style = style_from_dict({
        Token.QuestionMark:     '#E91E63 bold',
        Token.Selected:         '#673AB7 bold',
        Token.Instruction:      '',                # default
        Token.Answer:           '#2196f3 bold',
        Token.Question:         '',
    })

    """
    Dialogue
    """
    # 1. Dependency check
    dp_check = [{
                    'type': 'confirm',
                    'name': 'ready',
                    'message': 'Are you ready to proceed?',
                    'default': True
                }]

    if not prompt(dp_check, style=style)['ready']:
        smart_print_color("\n     Operation cancelled\n", "red")
        sys.exit()

    smart_print_color("\n     ~Running dependency checks", "yellow")
    shell("pip3 install --upgrade anchorage")
    init()
    smart_print_color("     ~Everything in order!", "green")
    smart_print_color("     ~Edit configurations in ~/.anchorage/config.toml\n", "green")

    # 2. Browser choice
    cfg = read_config()

    brow_choice = [{
                    'type': 'list',
                    'name': 'browser',
                    'message': f'Please choose the browser from which to archive all bookmarks.',
                    'default': True,
                    'choices': list(cfg.keys())[:-1],
                    'filter': lambda n: n.lower()
                   }]

    bmk_dict = load(path(prompt(brow_choice, style=style)['browser']))
    newline()

    # 3. Bookmark filter
    #     - Local files
    #     - By string match
    #     - By substring match
    #     - By regex match
    bk_filt = [{
                    'type': 'checkbox',
                    'name': 'drop',
                    'message': f'If desired, specify which elements to exclude from the archiving process.',
                    'choices': [
                        {
                            'name': 'Local files'
                        },
                        {
                            'name': 'Match string'
                        },
                        {
                            'name': 'Match substring'
                        },
                        {
                            'name': 'Regex'
                        },
                    ],
                    'filter': lambda lst: [n.lower() if isinstance(n, str) else n for n in lst]
                  }]

    filter_kind = [{
                    'type': 'checkbox',
                    'name': 'kind',
                    'message': f'Specify filter targets.',
                    'choices': [
                        {
                            'name': 'Directories'
                        },
                        {
                            'name': 'Names'
                        },
                        {
                            'name': 'URLs'
                        },
                    ],
                    'filter': lambda lst: [n.lower() if isinstance(n, str) else n for n in lst]
                  }]

    filter_input = lambda terms: prompt([{
                                            'type': 'input',
                                            'name': 'input',
                                            'message': f'Enter a comma-separated list of the {terms} to filter.',
                                            'default': '',
                                            'filter': lambda string: string.split(",")
                                        }], style=style)['input']

    filter_regex = lambda terms: prompt([{
                                            'type': 'input',
                                            'name': 'input',
                                            'message': f'Enter regex formula to filter OUT {terms}.',
                                            'default': ''
                                         }], style=style)['input']

    drop_list = prompt(bk_filt, style=style)['drop']
    newline()

    drop_dirs       = None
    drop_names      = None
    drop_urls       = None
    drop_dir_subs   = None
    drop_name_subs  = None
    drop_url_subs   = None
    drop_dir_regex  = None
    drop_name_regex = None
    drop_url_regex  = None

    if 'match string' in drop_list:
        kind = prompt(filter_kind, style=style)['kind']
        newline()
        if 'directories' in kind:
            drop_dirs = filter_input('directories')
            newline()
        if 'names' in kind:
            drop_names = filter_input('names')
            newline()
        if 'urls' in kind:
            drop_urls = filter_input('URLs')
            newline()
    if 'match substring' in drop_list:
        kind = prompt(filter_kind, style=style)['kind']
        newline()
        if 'directories' in kind:
            drop_dir_subs = filter_input('directory substrings')
            newline()
        if 'names' in kind:
            drop_name_subs = filter_input('name substrings')
            newline()
        if 'urls' in kind:
            drop_url_subs = filter_input('URL substrings')
            newline()
    if 'regex' in drop_list:
        kind = prompt(filter_kind, style=style)['kind']
        newline()
        if 'directories' in kind:
            drop_dir_regex = filter_regex('bookmark directories')
            newline()
            expr_check(drop_dir_regex)
        if 'names' in kind:
            drop_name_regex = filter_regex('bookmark names')
            newline()
            expr_check(drop_name_regex)
        if 'urls' in kind:
            drop_url_regex = filter_regex('bookmark URLs')
            newline()
            expr_check(drop_url_regex)

    smart_print_color("     ~Applying filters to bookmark collection", "yellow")

    bmk = bookmarks(bmk_dict,
                    drop_local_files='local files' in drop_list,
                    drop_dirs=drop_dirs,
                    drop_names=drop_names,
                    drop_urls=drop_urls,
                    drop_dirs_subs=drop_dir_subs,
                    drop_names_subs=drop_name_subs,
                    drop_urls_subs=drop_url_subs,
                    drop_dirs_regex=drop_dir_regex,
                    drop_names_regex=drop_name_regex,
                    drop_urls_regex=drop_url_regex,
                    )

    smart_print_color(f"     ~Done! Found {len(bmk.bookmarks)} bookmarks in {bmk.n_dirs} directories.\n", "green")

    # 4. Local or online archive
    archive_choice = [{
                    'type': 'list',
                    'name': 'archive',
                    'message': f'Choose an archiving method.',
                    'choices': ['Local (ArchiveBox)', 'Online'],
                    'filter': lambda n: n.split(' ')[0].lower()
                     }]

    archive = prompt(archive_choice, style=style)['archive']

    if archive == 'online':         # Anchor online
        anchor_online(bmk)

    if archive == 'local':          # Anchor locally

        dir_prompt = [{             # Archive directory input
                          'type': 'input',
                          'name': 'dir',
                          'message': 'Enter the full path of the archive directory (default: ./anchor).',
                          'default': './anchor'
                     }]
        archive_dir = prompt(dir_prompt, style=style)['dir']

        anchor_locally(bmk, archive_dir)



import os
import shutil
import unittest

from anchorage import add_local as add, server
from anchorage.anchor_tools.local import setup, init


class Tests(unittest.TestCase):

    def test_all(self):
        t_setup()
        t_init()
        t_add()
        t_server()


test_archive = r"C:\Users\xXY4n\Projects\Anchorage\tests\test_archive"


def t_setup():
    print("LOCAL ARCHIVE TESTS: SETUP")
    if os.path.isdir(test_archive):
        shutil.rmtree(test_archive)
    setup(archive=test_archive)


def t_init():
    print("LOCAL ARCHIVE TESTS: INITIALIZATION")
    init(archive=test_archive)


def t_add():
    print("LOCAL ARCHIVE TESTS: ADDITION")
    add(url="http://larsblackmore.com/AcikmeseBlackmoreAutomatica10.pdf",
        archive=test_archive)


def t_server():
    print("LOCAL ARCHIVE TESTS: SERVER")
    server(archive=test_archive, instakill=True)



import os
import sys
import shutil
import webbrowser

from Alexandria.general.console import print_color

from anchorage.anchor_utils.shell import shell, bckgr, raise_error, error, suppress_stdout
from anchorage.anchor_utils.aesthetic import smart_print_color


def archivebox(command_str):
    """
    Shortcut for the shell command:

        docker-compose run archivebox {command}
    """
    return shell(f"docker-compose run archivebox {command_str}")


def setup(archive):
    """
    Download ArchiveBox docker-compose.yml

    :param archive: Full path of archive directory
    """
    if not os.path.isdir(archive):
        os.mkdir(archive)
        os.chdir(archive)
    else:
        os.chdir(archive)

    docker_compose_yaml = "https://raw.githubusercontent.com/ArchiveBox/ArchiveBox/master/docker-compose.yml"
    if not os.path.isfile(f"{archive}/docker-compose.yml"):
        try:
            shell(f"curl -O {docker_compose_yaml}")
        except error():
            sys.exit("GNU wget and curl unavailable. Install any to proceed.")


def init(archive=None):
    """
    Creates ArchiveBox archive at dest

    :param archive: Full path of archive directory
    """
    if not isinstance(archive, type(None)):
        os.chdir(archive)

    archivebox("init --setup")


def create_archive(archive, overwrite=False):
    """
    Creates local archive.

    :param archive: Archive directory.
    :param overwrite: Overwrite directory if it already exists.
    """
    if os.path.isdir(archive):
        if overwrite:
            shutil.rmtree(archive)
            setup(archive=archive)
    else:
        setup(archive=archive)

    init(archive=archive)


def add(url, archive=None):
    """
    Adds website to archive in current directory, or a specified archive

    :param url: URL of website to be archived
    :param archive: Full path of archive directory
    """
    if not isinstance(archive, type(None)):
        os.chdir(archive)

    try:
        archivebox(f"add {url}")
    except:
        print("Error archiving: ", end="")
        print_color(url, "red")


def server(archive=None, browser_path=None, instakill=False):
    """
    Starts ArchiveBox server and opens it in browser of choice
    """

    if not isinstance(archive, type(None)):
        os.chdir(archive)

    p = bckgr("docker-compose up")  # Server

    browser = webbrowser.get(browser_path) if not isinstance(browser_path, type(None)) else webbrowser
    browser.open("http://127.0.0.1:8000")

    if not instakill:
        input("Click any key to close the server.")

    p.terminate()


def docker_check():
    """
    Attempts to run the `docker` command, and stops the program if an error is raised.
    """
    with suppress_stdout():
        # Docker install check
        try:
            raise_error('docker')
        except error() as e:
            smart_print_color(f"Error: attempt to run 'docker' command in your shell failed. "
                              f"       Install Docker if you don't have it installed."
                              f"       Check your Docker installation by running 'docker --help' in your shell.",
                              'red')
            sys.exit()


def archivebox_check():
    """
    Attempts to run the `archivebox` command, and stops the program if an error is raised.
    """
    with suppress_stdout():
        # ArchiveBox install check
        try:
            raise_error('archivebox')
        except error() as e:
            smart_print_color(f"Error: attempt to run 'archivebox' command in your shell failed. "
                              f"       Install ArchiveBox if you don't have it installed."
                              f"       Check your ArchiveBox installation by running 'archivebox' in your shell.",
                              'red')
            sys.exit()



import re
import lz4.block


def JSONLZ4_to_JSON(path):
    """
    Turns a Firefox JSONLZ4 file into a string readable by `rapidjson`
    and with the same bookmark dictionary keys as those of Chromium
    browsers.

    :param path: Path to the JSONLZ4 file.
    """
    stream = open(path, 'rb')
    stream.read(8)                  # Skip b"mozLz40\0" header
    string = lz4.block.decompress(stream.read()).decode('utf-8')
    string = re.sub('title', 'name', string)
    string = re.sub('uri', 'url', string)
    return string



import os
import sys
import subprocess
from contextlib import contextmanager


def shell(command):
    """
    Executes command in shell.

    :param command: command string
    """
    return subprocess.run(command, shell=True, capture_output=True)


def raise_error(command):
    """
    Executes command in shell and raises an error if the command outputs non-zero exit code.

    :param command: command string
    """
    return subprocess.check_output(command, shell=True)


def bckgr(command):
    """
    Run shell command as background process.

    :param command: command string
    """
    return subprocess.Popen(command, shell=True)


def error():
    """
    :return: subprocess.CalledProcessError
    """
    return subprocess.CalledProcessError


@contextmanager
def suppress_stdout():
    """
    Suppress console output.
    """
    with open(os.devnull, "w") as devnull:
        old_stdout = sys.stdout
        sys.stdout = devnull
        try:
            yield
        finally:
            sys.stdout = old_stdout



import os
import toml

from anchorage.anchor_infrs.config import config
from anchorage.anchor_utils.system import home


def init(overwrite=True):
    """
    Initialize the Anchorage library

    1. Create ~/anchorage
    2. Save config.toml in /anchorage

    :param overwrite: Overwrite current config.toml if found
    """

    anchorage_dir = home() + '/.anchorage'
    anchorage_cfg = home() + '/.anchorage/config.toml'
    create_cfg = lambda: open(anchorage_cfg, 'w').write(config)

    install_status = check_install()

    if install_status == 0 and overwrite:
        create_cfg()
    if install_status == 1:
        create_cfg()
    if install_status == 2:
        os.mkdir(anchorage_dir)
        create_cfg()

    return anchorage_cfg


def check_install():
    """
    Check if

    1. ~/anchorage/ exists
    2. config.toml exists
    """
    anchorage_dir = home() + '/.anchorage'
    anchorage_cfg = home() + '/.anchorage/config.toml'

    if os.path.isdir(anchorage_dir):
        if os.path.isfile(anchorage_cfg) and os.stat(anchorage_cfg).st_size > 0:
            return 0
        else:
            return 1
    else:
        return 2


def read_config():
    anchorage_cfg = home() + '/.anchorage/config.toml'
    return toml.load(anchorage_cfg)



config = '# Welcome to the\n' \
         '#     /\              | |                                      | (_)\n' \
         '#    /  \   _ __   ___| |__   ___  _ __ __ _  __ _  ___     ___| |_\n' \
         '#   / /\ \ |  _ \ / __|  _ \ / _ \|  __/ _  |/ _  |/ _ \   / __| | |\n' \
         '#  / ____ \| | | | (__| | | | (_) | | | (_| | (_| |  __/  | (__| | |\n' \
         '# /_/    \_\_| |_|\___|_| |_|\___/|_|  \__,_|\__, |\___|   \___|_|_|\n' \
         '#                                            __/ |                  \n' \
         '#                                           |___/                   \n' \
         '#                                                 Configuration file\n' \
         '#\n' \
         '# Below you can see the path to the Bookmarks JSON/JSONLZ4 file for a\n' \
         '# series of browsers.\n' \
         '# \n' \
         '# Set your default browser to one of the available to skip browser\n' \
         '# choice when using the Anchorage CLI. \n' \
         '# \n' \
         '# You can add new browsers and the path to their bookmarks file. You \n' \
         '# will then be able to set the new browser as the default, or choose\n' \
         '# it in the Anchorage CLI.\n' \
         '# \n' \
         '# If using a custom browser **it must comply with the Chromium or \n' \
         '# Firefox bookmark storage conventions (i.e. dictionary key naming)**.\n' \
         '# If this is not the case, export your bookmarks to a supported browser\n' \
         '# and proceed.\n' \
         '#                                           Antonio Lopez Rivera, 2021\n' \
         '\n' \
         '[chrome]\n' \
         'linux = "~/.config/google-chrome/Default/Bookmarks"\n' \
         'macos = "~/Library/Application Support/Google/Chrome/Default/Bookmarks"\n' \
         'windows = "Local/Google/Chrome/User Data/Default/Bookmarks"\n' \
         '\n' \
         '[edge]\n' \
         'linux = "?"\n' \
         'macos = "?"\n' \
         'windows = "Local/Microsoft/Edge/User Data/Default/Bookmarks"\n' \
         '\n' \
         '[firefox]\n' \
         'linux = "?"\n' \
         'macos = "?"\n' \
         'windows = "Roaming/Mozilla/Firefox/Profiles/mfeew61f.default-release/bookmarkbackups/" \n' \
         '\n' \
         '["edge beta"]\n' \
         'linux = "?"\n' \
         'macos = "?"\n' \
         'windows = "Local/Microsoft/Edge Beta/User Data/Default/Bookmarks"\n' \
         '\n' \
         '[defaults]\n' \
         'browser = "edge beta"\n' \



import re
import sys
from anchorage.anchor_utils.aesthetic import smart_print_color


def expr_check(expr):
    """
    Checks a given regex expression for correctness by compiling it
    and stopping execution if an error is raised.

    :param expr: Regex expression to be checked
    """
    try:
        re.compile(expr)
    except re.error:
        smart_print_color("\n     Incorrect regex formula (regex compile error)\n", "red")
        sys.exit()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="anchorage",
    version="0.9.0.3",
    author="Antonio Lopez Rivera",
    author_email="antonlopezr99@gmail.com",
    description="Anchor your little piece of internet.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/antonlopezr/anchorage",
    scripts=['run/anchorage'],
    packages=setuptools.find_packages(),
    install_requires=[
        "numpy",
        "rapidjson",
        "toml",
        "Python-Alexandria",
        "archivenow",
        "internetarchive",
        "tqdm",
        "pyfiglet",
        "PyInquirer",
        "prompt-toolkit==1.0.14",
        "lz4"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)



import unittest

from anchorage import add_online as add


class OnlineArchiveTests(unittest.TestCase):

    def test_add(self):
        print(add("http://paulgraham.com/think.html", "--ia", overwrite=True)[0])

    def test_existing(self):
        assert add("http://paulgraham.com/think.html", "--ia") == "Bookmark already present in the Internet Archive"
